import logging
import json

galaxy_alls = {'alpine', 'archlinux'}

galaxy_os_name = {
    "alpine": "Alpine",
    "archlinux": "ArchLinux",
    "debian": "Debian",
    "fedora": "Fedora",
    "rockylinux": "EL",
    "ubuntu": "Ubuntu"
}

latest_pairs = {
    "alpine": {"3.20", "3.21"},
    "archlinux": {"latest"},
    "debian": {"bookworm", "bullseye"},
    "fedora": {"40", "41"},
    "rockylinux": {"8", "9"},
    "ubuntu": {"jammy", "noble"}
}

upgrades = {
    "alpine": {
        "3.21": {"3.21"},
        "edge": {"edge"}
    },
    "debian": {
        "bookworm": {"bookworm"}
    },
    "fedora": {
        "41": {"41"}
    },
    "rockylinux": {
        "9": {"9"}
    },
    "ubuntu": {
        "noble": {"noble"}
    }
}

def upgrade(pm_in):
    by_os = {}
    # apply upgrades and flatten to set
    for p in pm_in:
        os = p["OS"]
        bo = by_os.setdefault(os, set())

        os_ver = p["OS_VER"]

        if os in upgrades.keys():
            ups = upgrades[os]
            if os_ver in ups:
                bo |= ups[os_ver]
            else:
                bo |= latest_pairs[os]
        else:
            bo |= {os_ver}


    pm_out = []
    # reinflate sets
    for os in sorted(by_os.keys()):
        for os_ver in sorted(by_os[os]):
            pm_out.append({"OS": os, "OS_VER": os_ver})
    return pm_out

def from_dcb_osl(osl):
    pm = []
    for o in osl:
        s = o.split('_')
        pm.append({"OS": s[0], "OS_VER" : s[1]})
    return upgrade(pm)

def render_platforms(pm):
    by_os = {}
    for p in pm:
        os = p['OS']
        os_name = galaxy_os_name[os]
        bo = by_os.setdefault(os_name, set())
        if os in galaxy_alls:
            bo |= {"all"}
        else:
            bo |= {p["OS_VER"]}

    # reinflate sets
    platforms = []
    for os_name in sorted(by_os.keys()):
        platforms.append({
            "name": os_name,
            "versions": sorted(by_os[os_name])
        })
    return platforms



import argparse
import logging
import os
import sys

import ansiblegalaxylocaldeps.deps as deps
import ansiblegalaxylocaldeps.dump as dump
import ansiblegalaxylocaldeps.loggingsetup as loggingsetup
import ansiblegalaxylocaldeps.slurp as slurp

def run(role_dir: str) -> None:
  mm = slurp.slurp_meta_main(role_dir)
  if 'dependencies' in mm:
    dump.dump_requirements_yml(
      role_dir,
      deps.extract_dependencies(mm['dependencies'])
    )

def main() -> None:
  loggingsetup.go()

  parser = argparse.ArgumentParser(
    description='extracts dependencies from meta/main.yml and writes out meta/requirements.yml'
  )
  parser.add_argument('roledirs', nargs='*', default=[os.getcwd()])
  args = parser.parse_args()
  for roledir in args.roledirs:
    run(roledir)



from unittest import TestCase
from yaml import load, dump
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from ansiblegalaxylocaldeps.platform_matrix import from_dcb_osl, upgrade

class TestDeps(TestCase):

    def test_from_dcb_osl(self):
        y = load(
"""
---
- alpine_3.21
- ubuntu_noble
""",
            Loader=Loader
        )
        pm = from_dcb_osl(y)
        self.assertEqual(len(pm), 2, 'count from converted dcb-os.yml')
        self.assertEqual(pm[0]["OS"], "alpine")
        self.assertEqual(pm[0]["OS_VER"], "3.21")
        self.assertEqual(pm[1]["OS"], "ubuntu")
        self.assertEqual(pm[1]["OS_VER"], "noble")

    def test_upgrade(self):
        pm = upgrade([
            {
                "OS": "alpine",
                "OS_VER": "3.19"
            },
            {
                "OS": "alpine",
                "OS_VER": "3.20"
            }
        ])
        self.assertEqual(len(pm), 2, 'count from converted dcb-os.yml')
        self.assertEqual(pm[0]["OS"], "alpine")
        self.assertEqual(pm[0]["OS_VER"], "3.20")
        self.assertEqual(pm[1]["OS"], "alpine")
        self.assertEqual(pm[1]["OS_VER"], "3.21")



from unittest import TestCase
from yaml import load, dump
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from ansiblegalaxylocaldeps.deps import effkey, extract_dependencies

class TestDeps(TestCase):

    def test_effkey(self):
        self.assertEqual(effkey({'role': 'a', 'src' : 'b'}), 'role', 'EffKey role before src')
        self.assertEqual(effkey({'src': 'a'}), 'src', 'EffKey only src')
        self.assertIsNone(effkey({'foo': 'bar'}), 'EffKey neither src nor role')

    def test_extract_dependencies(self):
        y = load(
"""
---
- role: test-role-1
- src: test-role-2
- name: test-role-3
""",
                 Loader=Loader)
        o = extract_dependencies(y)
        self.assertEqual(len(o), 3, 'count of simple extract_dependencies')
        self.assertEqual(o[0]['name'], 'test-role-1', 'simple extracted role name (role)')
        self.assertEqual(o[1]['name'], 'test-role-2', 'simple extracted role name (src)')
        self.assertEqual(o[2]['name'], 'test-role-3', 'simple extracted role name (name)')


    def test_extract_dependencies_with_version(self):
        y = load(
"""
---
- role: test-role-1
  version: v1.0.0
- name: test-role-2
  version: v2.0.0
""",
            Loader=Loader)
        o = extract_dependencies(y)
        self.assertEqual(len(o), 2, 'count of extract_dependencies with versions')
        self.assertEqual(o[0]['name'], 'test-role-1', 'extracted role name (1)')
        self.assertEqual(o[0]['version'], 'v1.0.0', 'extracted role version (1)')
        self.assertEqual(o[1]['name'], 'test-role-2', 'extracted role name (2))')
        self.assertEqual(o[1]['version'], 'v2.0.0', 'extracted role version (2)')



import logging

from typing import Union

import ansiblegalaxylocaldeps.finder as finder

def effkey(d) -> Union[str, None]:
  if 'name' in d:
    return 'name'
  elif 'role' in d:
    return 'role'
  elif 'src' in d:
    return 'src'
  else:
    return None

def extract_dependencies(requirements_yml):
  """extract dependencies from a requirements.yml yaml data"""
  log = logging.getLogger('ansible-galaxy-local-deps.deps.extract_dependencies')
  o = []
  for r in requirements_yml:
    key = effkey(r)
    if key:
      r['name'] = r[key]
      if key != 'name':
        r.pop(key)
      o.append(r)
    else:
      log.warn('ignoring dependency: {0}'.format(r))
  return o



import logging
import os
import json
from yaml import load
try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

from ansiblegalaxylocaldeps import finder as finder

def slurp_yml(role_dir: str, f: str):
    log = logging.getLogger('ansible-galaxy-local-deps.slurp.slurp_yml')
    fq = finder.find(role_dir, f)
    if fq:
        log.info('found {0}. slurping...'.format(fq))
        with open(fq, 'r') as ifq:
            return load(ifq, Loader=Loader)
    return None

def slurp_json(role_dir: str, f: str):
    log = logging.getLogger('ansible-galaxy-local-deps.slurp.slurp_json')
    fq = finder.find(role_dir, f)
    if fq:
        log.info('found {0}. slurping...'.format(fq))
        with open(fq, 'r') as ifq:
            return json.load(ifq)
    return None

def slurp_meta_main_yml(role_dir: str):
    return slurp_yml(
        role_dir,
        os.path.join('meta', 'main.yml')
    )

def slurp_meta_requirements_yml(role_dir: str):
    return slurp_yml(
        role_dir,
        os.path.join('meta', 'requirements.yml')
    )

def slurp_test_requirements_yml(role_dir: str):
    return slurp_yml(role_dir, 'test-requirements.yml')

def slurp_dcb_os_yml(role_dir: str):
    return slurp_yml(role_dir, 'dcb-os.yml')

def slurp_script_yml(role_dir: str):
    return slurp_yml(role_dir, 'script.yml')

def slurp_platform_matrix_json(role_dir: str):
    return slurp_json(role_dir, 'platform-matrix-v1.json')



import argparse
import logging
import os
from typing import List

import ansiblegalaxylocaldeps.dump as dump
import ansiblegalaxylocaldeps.loggingsetup as loggingsetup
import ansiblegalaxylocaldeps.slurp as slurp
import ansiblegalaxylocaldeps.platform_matrix as platform_matrix

def build_yml(ver: str="v1"):
    return {
        'on': 'push',
        'jobs': {
            'bake-ansible-images-v1': {
                'uses': 'andrewrothstein/.github/.github/workflows/bake-ansible-images-{}.yml@develop'.format(ver)
            }
        }
    }

def render_meta_main(role_dir: str, pm):
    log = logging.getLogger('ansible-galaxy-local-deps.gengithubactions.render_meta_main')
    mm = slurp.slurp_meta_main_yml(role_dir)

    # flatten the license list
    l = mm['galaxy_info']['license']
    if isinstance(l, list):
        mm['galaxy_info']['license'] = l[0]

    min_ansible_version = mm['galaxy_info']['min_ansible_version']
    if isinstance(min_ansible_version, float):
        mm['galaxy_info']['min_ansible_version'] = str(min_ansible_version)

    mm['galaxy_info']['platforms'] = platform_matrix.render_platforms(pm)

    if 'namespace' not in mm['galaxy_info']:
        mm['galaxy_info']['namespace'] = 'andrewrothstein'

    if 'role_name' not in mm['galaxy_info']:
        log.info('computing role_name from role directory: {}'.format(role_dir))
        d = os.path.basename(role_dir)
        if d.startswith('ansible-'):
            rn = d.removeprefix('ansible-')
            log.info('computed role_name: {}'.format(rn))
            mm['galaxy_info']['role_name'] = rn

    dump.dump_meta_main_yml(role_dir, mm)

def upgrade_platform_matrix(
        role_dir: str
):
    dcb_os = os.path.join(role_dir, "dcb-os.yml")
    pm = None
    if os.path.exists(dcb_os):
        pm = platform_matrix.from_dcb_osl(
            slurp.slurp_dcb_os_yml(role_dir)
        )
        os.remove(dcb_os)
    else:
        pm = platform_matrix.upgrade(
            slurp.slurp_platform_matrix_json(role_dir)
            )
    dump.dump_platform_matrix_json(role_dir, pm)
    render_meta_main(role_dir, pm)

def mksubdirs(role_dir: str, subs: List[str]):
    d = role_dir
    for s in subs:
        d = os.path.join(d, s)
        if not os.path.isdir(d):
            os.mkdir(d, 0o755)

def main():
    loggingsetup.go()
    parser = argparse.ArgumentParser(
        description='generates a .github/workflows/build.yml for building/testing Ansible roles with docker buildx bake'
    )
    log = logging.getLogger('ansible-galaxy-local-deps.gengithubactions.main')
    parser.add_argument('roledirs', nargs='*', default=[os.getcwd()])
    parser.add_argument('--ver', default="v1")
    args = parser.parse_args()
    for role_dir in args.roledirs:
        mksubdirs(role_dir, [".github", "workflows"])
        upgrade_platform_matrix(role_dir)
        dump.dump_github_actions_build_yml(role_dir, build_yml(args.ver))
        dump.dump_gitignore(role_dir)



import argparse
import logging
import os
import sys

import ansiblegalaxylocaldeps.deps as deps
import ansiblegalaxylocaldeps.dump as dump
import ansiblegalaxylocaldeps.loggingsetup as loggingsetup
import ansiblegalaxylocaldeps.slurp as slurp

def adjust_role(role_map, ek: str, r: str, v: str):
  if ek != 'name':
    role_map['name'] = role_map[ek]
    role_map.pop(ek)
  if v is None:
    role_map['name'] = r
  else:
    role_map['name'] = r
    role_map['version'] = v
  return role_map

def rewrite(
    r_yml,
    from_role: str,
    from_ver: str,
    to_role: str,
    to_ver: str
):
  if r_yml is None:
    return None

  o = []
  modified = False
  for r in r_yml:
    ek = deps.effkey(r)
    if ek is not None and from_role == r[ek]:
      if from_ver is None:
        o.append(adjust_role(r, ek, to_role, to_ver))
        modified = True
      elif 'version' in r and r['version'] == from_ver:
        o.append(adjust_role(r, ek, to_role, to_ver))
        modified = True
      else:
        o.append(r)
    else:
      o.append(r)
  return o if modified else None

def rewrite_meta_requirements_yml(
    role_dir: str,
    from_role: str,
    from_ver: str,
    to_role: str,
    to_ver: str
):
  modified = rewrite(
    slurp.slurp_meta_requirements_yml(role_dir),
    from_role,
    from_ver,
    to_role,
    to_ver
  )
  if modified:
    dump.dump_meta_requirements_yml(role_dir, modified)

def rewrite_test_requirements_yml(
    role_dir: str,
    from_role: str,
    from_ver: str,
    to_role: str,
    to_ver: str
):
  modified = rewrite(
    slurp.slurp_test_requirements_yml(role_dir),
    from_role,
    from_ver,
    to_role,
    to_ver
  )
  if modified:
    dump.dump_test_requirements_yml(role_dir, modified)

def run(
    role_dir: str,
    from_role: str,
    from_ver: str,
    to_role: str,
    to_ver: str
) -> None:
  log = logging.getLogger('ansible-galaxy-local-deps-change-dep')

  log.info(
    "changing role {0} to {1}".format(
      from_role if from_ver is None else "{0}:{1}".format(from_role, from_ver),
      to_role if to_ver is None else "{0}:{1}".format(to_role, to_ver)
    )
  )
  rewrite_meta_requirements_yml(
    role_dir,
    from_role,
    from_ver,
    to_role,
    to_ver
  )
  rewrite_test_requirements_yml(
    role_dir,
    from_role,
    from_ver,
    to_role,
    to_ver
  )

def main() -> None:
  loggingsetup.go()

  parser = argparse.ArgumentParser(
    description="modified dependencies in meta/requirements.yml and test-requirements.yml files"
  )
  parser.add_argument('roledirs', nargs='*', default=[os.getcwd()])
  parser.add_argument('--role')
  parser.add_argument('--fromver', default=None)
  parser.add_argument('--torole', default=None)
  parser.add_argument('--tover', default=None)
  args = parser.parse_args()
  for roledir in args.roledirs:
    run(
      roledir,
      args.role,
      args.fromver,
      args.role if args.torole is None else args.torole,
      args.tover
    )



import logging
import os

from typing import Union

def find(role_dir: str, f: str) -> Union[str, None]:
    log = logging.getLogger('ansible-galaxy-local-deps.finder.find')
    fq = os.path.join(role_dir, f)
    log.info('looking for {0}...'.format(fq))
    return fq if os.path.isfile(fq) else None

def find_meta_main(role_dir: str) -> Union[str, None]:
    return find(role_dir, os.path.join('meta', 'main.yml'))

def find_dcb_os(role_dir: str) -> Union[str, None]:
    return find(role_dir, 'dcb-os.yml')

def find_gha_buildyml(role_dir: str) -> Union[str, None]:
    return find(role_dir, os.path.join(".github", "workflows", "build.yml"))



from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(name='ansible-galaxy-local-deps',
      version='0.5.3',
      description='CLI for interacting with Ansible roles and their CIs',
      url='http://github.com/andrewrothstein/ansible-galaxy-local-deps',
      author='Andrew Rothstein',
      author_email='andrew.rothstein@gmail.com',
      license='MIT',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      python_requires='>=3.5',
      install_requires=['PyYAML'],
      extras_require={
          "test": ['nose2']
      },
      long_description=long_description,
      long_description_content_type='text/markdown',
      zip_safe=False,
      entry_points={
        'console_scripts': [
            'ansible-galaxy-local-deps-install=ansiblegalaxylocaldeps.installdeps:main',
            'ansible-galaxy-local-deps-write=ansiblegalaxylocaldeps.writedeps:main',
            'gengithubactions=ansiblegalaxylocaldeps.gengithubactions:main',
            'ansible-galaxy-local-deps-change-dep=ansiblegalaxylocaldeps.changedep:main',
        ]
      }
      )



import argparse
import logging
import os
from subprocess import check_call
import sys

import ansiblegalaxylocaldeps.deps as deps
import ansiblegalaxylocaldeps.loggingsetup as loggingsetup
import ansiblegalaxylocaldeps.slurp as slurp

def install_role(r: str, v: str=None) -> None:
    log = logging.getLogger('ansible-galaxy-local-deps.installdeps.install')
    log.info('installing {0} version {1}...'.format(r, v))
    p = ','.join([r, v]) if v is not None else r
    check_call(['ansible-galaxy', 'install', '-f', p])


def install_all(y) -> None:
    if y is not None:
        for d in y:
            efk = deps.effkey(d)
            if efk is not None:
                install_role(
                    d[efk],
                    d['version'] if 'version' in d else None
                )
            else:
                log.info('ignoring key {}'.format(d))
    else:
        log.info('no dependencies')

def run(role_dir: str) -> None:
    log = logging.getLogger(
        'ansible-galaxy-local-deps.installdeps.run'
    )
    install_all(
        deps.extract_dependencies(
            slurp.slurp_meta_requirements(role_dir)
        )
    )
    install_all(
        deps.extract_dependencies(
            slurp.slurp_test_requirements(role_dir)
        )
    )


def main():
    loggingsetup.go()

    parser = argparse.ArgumentParser(
        description='uses ansible-galaxy to install all dependencies from test-requirements.yml and meta/requirements.yml'
    )
    parser.add_argument('roledirs', nargs='*', default=[os.getcwd()])
    args = parser.parse_args()
    for roledir in args.roledirs:
        run(roledir)



import logging
import os
import sys
import json
import yaml

class IndentDumper(yaml.Dumper):
    def increase_indent(self, flow=False, indentless=False):
        return super(IndentDumper, self).increase_indent(flow, False)

def dump_txt(role_dir: str, f: str, t: str) -> None:
    log = logging.getLogger('ansible-galaxy-local-deps.dump.dump_txt')
    of = os.path.join(role_dir, f)
    log.info('writing out {}...'.format(of))
    with open(of, 'w') as w:
        w.write(t)

def dump_yml(role_dir: str, f: str, y) -> None:
    log = logging.getLogger('ansible-galaxy-local-deps.dump.dump_yml')
    of = os.path.join(role_dir, f)
    log.info('writing out {}...'.format(of))
    with open(of, 'w') as s:
        yaml.dump(
            y,
            stream=s,
            explicit_start=True,
            Dumper=IndentDumper
        )

def dump_json(role_dir: str, f: str, j) -> None:
    log = logging.getLogger('ansible-galaxy-local-deps.dump.dump_json')
    of = os.path.join(role_dir, f)
    log.info('writing out {}...'.format(of))
    with open(of, 'w') as s:
        json.dump(j, s, indent=2)

def dump_meta_main_yml(role_dir: str, y) -> None:
    dump_yml(role_dir, os.path.join('meta', 'main.yml'), y)

def dump_meta_requirements_yml(role_dir: str, y) -> None:
    dump_yml(role_dir, os.path.join('meta', 'requirements.yml'), y)

def dump_requirements_yml(role_dir: str, y) -> None:
    dump_yml(role_dir, 'requirements.yml', y)

def dump_requirements_txt(role_dir: str, t: str) -> None:
    dump_txt(role_dir, 'requirements.txt', t)

def dump_dcb_os_yml(role_dir: str, y) -> None:
    dump_yml(role_dir, 'dcb-os.yml', y)

def dump_platform_matrix_json(role_dir: str, j) -> None:
    dump_json(role_dir, 'platform-matrix-v1.json', j)

def dump_test_requirements_yml(role_dir: str, y) -> None:
    dump_yml(role_dir, 'test-requirements.yml', y)

def dump_github_actions_build_yml(role_dir: str, y) -> None:
    dump_yml(role_dir, os.path.join('.github', 'workflows', 'build.yml'), y)

def dump_gitignore(role_dir: str):
    dump_txt(
        role_dir,
        ".gitignore",
        "\n".join([
            '**~'
            '*.retry',
            'Dockerfile.*',
            'requirements.yml',
            '!meta/requirements.yml',
            '**/*undo-tree*'
        ]))
